import os
import sys
import veux
import numpy as np
import opensees.openseespy as ops
from xsection.library import from_aisc

def _test_opensees(shape, section, center=None):
    model = ops.Model(ndm=3, ndf=6)
    E    = 1e3 #29000*ksi,
    nu   = 0.3
    G = 0.5*E/(1+nu)
    model.material("ElasticIsotropic", 1, E, nu, 0.0)
    model.section("ShearFiber", 1)

    if section == "fiber":
        for fiber in shape.fibers(center=center):
            y, z = fiber.location
           #model.fiber(y, z, fiber.area, warp=[fiber.warp[0], fiber.warp[1], [0,0,0]], material=1, section=1)
            model.fiber(**fiber, material=1, section=1)
    else:
        print("Section = Elastic")
        cnn = shape.torsion.cnn()
        cnm = shape.torsion.cnm()
#       cnw = shape.torsion.cnw()
        cnv = shape.torsion.cnv()
        cmm = shape.torsion.cmm()
        cww = shape.torsion.cww()
        cmv = shape.torsion.cmv()
        cmw = shape.torsion.cmw()
        A = cnn[0,0]
        model.section("ElasticFrame", 1,
                        E=E,
                        G=G,
                        A=A,
                        Ay=1*A,
                        Az=1*A,
                        Qy=cnm[0,1],
                        Qz=cnm[2,0],
                        Iy=cmm[1,1],
                        Iz=cmm[2,2],
#                       Iyz=-cmm[1,2],
                        J =shape.torsion.torsion_constant(),
                        Cw= cww[0,0],
#                       Rw= 0, #cnw[0,0], # this is pretty much always 0.0
                        Ry= cnv[1,0],
                        Rz= cnv[2,0],
                        Sa= cmv[0,0],
                        Sy= cmw[1,0],
                        Sz= cmw[2,0]
        )
        del cnn, cnm, cnv, cmm, cww, cmv, cmw

    tangent = model.invoke("section", 1, [
                           "update  "+" ".join(["0"]*12)+";",
                           "tangent"
            ])

    n = int(np.sqrt(len(tangent)))
    Ks = np.array(tangent).reshape(n,n).T
#   plt.spy(Ks)
#   plt.imshow(Ks,interpolation='none',cmap='binary')
#   plt.show()
    # Ks = np.round(Ks, 4)
    # print(pd.DataFrame(Ks))

    cnn = Ks[:3,:3]
    cmm = Ks[3:6,3:6]
    cnm = Ks[:3,3:6]
    cnw = Ks[:3,6:9]
    cnv = Ks[:3,9:12]
    cmw = Ks[3:6,6:9]
    cmv = Ks[3:6,9:12]
    cww = Ks[6:9,6:9]
    cvv = Ks[9:12,9:12]
    # print(f"{cnm = }")
    # print(f"{cnw = }")

    s = ""
    tol=1e-14

    EA = cnn[0,0]
    A  = EA/E
    GA = G*A

    Qy = cnm[0,1] # int z
    Qz = cnm[2,0] # int y
    # Compute centroid
    cx, cy = float(Qz/GA), float(Qy/EA)
    cx, cy = map(lambda i: i if abs(i)>tol else 0.0, (cx, cy))


    Ivv = cvv[0,0]
    Isv = cmm[0,0] - Ivv

    s += f"""

  [nn]    Area               {A           :>10.4}
  [nm]    Centroid           {0.0         :>10.4}  {cx          :>10.4}, {cy          :>10.4}
  [nw|v]                     {cnw[0,0]/GA :>10.4}  {cnv[1,0]/GA :>10.4}, {cnv[2,0]/GA :>10.4}

  [mm]    Flexural moments   {cmm[0,0]/G  :>10.4}  {cmm[1,1]/E  :>10.4}, {cmm[2,2]/E  :>10.4}, {cmm[1,2]/E :>10.4}
  [mv|w]                     {cmv[0,0]/G  :>10.4}  {cmw[1,0]/E  :>10.4}, {cmw[2,0]/E  :>10.4}


  [ww]    Warping constant   {cww[0,0]/E  :>10.4}
          Torsion constant   {Isv/G       :>10.4}
  [vv]    Bishear            {Ivv/G       :>10.4}
    """
    print(s)

if __name__ == "__main__":
    c = "centroid"
    sec = from_aisc(sys.argv[1])
    d = sec.d
    shape = sec.create_shape()

    if c == "shear-center":
        shape = shape.translate(shape.torsion.shear_center())

    elif c == "centroid":
        shape = shape.translate(shape.torsion.centroid())

    else:
        shape = shape.translate(c)

    print(shape.summary())

    _test_opensees(shape,
                   section=os.environ.get("Section", "fiber"),
                   center =os.environ.get("Center", None)
    )

    # 1) create basic section
    basic = shape.linearize()

    field = shape.torsion.warping()

    # 3) view warping modes
    artist = veux.create_artist((shape.mesh.nodes, shape.mesh.cells()), ndf=1)

    field = {node: value/100.0 for node, value in enumerate(field)}

    artist.draw_surfaces(field = field, state=field)
    artist.draw_outlines()
    R = artist._plot_rotation

    artist.canvas.plot_vectors([[0,0,0] for i in range(3)], d/5*R.T)
    artist.canvas.plot_vectors([R@[*shape.torsion.shear_center(), 0] for i in range(3)], d/5*R.T)
    artist.draw_outlines()
    veux.serve(artist)

